﻿using System;
using UnityEngine;
using System.Collections;
using Random = UnityEngine.Random;

public enum EEnemySwoopState
{
    swooping_in = 0,
    maintaining,
    swooping_out,
}

public class EnemyBase : MonoBehaviour 
{
    Vector3 vActualPosition;

    public ShepAnim idleAnim;

    private ShepSpriteAnimator myAnimator;

    //How far down the screen should they move?
    public float fGoalYPosition;
    public bool bSwoopFromBehind;

    public Vector3 vMoveImpetus;

    public EEnemySwoopState swoopStateEnum;

    public bool bAvoidPlayerTruck;

    float fSwoopTimer;

    private PlayerTruck theTruck;
    private GameManager theGame;

    public GameObject Prefab_DecalOnDeath;

    private float fRandomDriverSluggishness;

    public float fCurrentHealth;
    public float fMaxHealth= 5.0f;
    public bool bIsDead = false;

    private Vector3 vImpetusFromOutside;

    //poorly tracked, highly inaccurate, do not trust for finance or medical purposes
    public int iNumWeaponLocksOnMe;

    public AudioSource audioSource;
    public AudioClip clip_onDeath;


    private float fChangeDirectionTimer;

	// Use this for initialization
	void Start () 
    {
        theTruck = GameObject.Find("PlayerTruck").GetComponent<PlayerTruck>();
        theGame = GameObject.Find("GameManager").GetComponent<GameManager>();

	    if (!bSwoopFromBehind)
	    {
	        fGoalYPosition = 3.25f + (-0.5f + 2.5f*Random.value);
	        vActualPosition = new Vector3(-3f + 6.0f*Random.value, 7.0f, 0.0f);
	        vMoveImpetus = new Vector3(0, -3, 0);
	    }
	    else
	    {
            fGoalYPosition = 1.0f + (-0.25f + 0.5f * Random.value);
	        vActualPosition = transform.position;
            vMoveImpetus = new Vector3(0, 3.8f + 0.4f * Random.value, 0);
	    }

	    if (vActualPosition.x > 0)
        {
            vMoveImpetus.x = -1.0f;
        }
        else
            vMoveImpetus.x = 1.0f;

	    myAnimator = GetComponent<ShepSpriteAnimator>();
        myAnimator.SetAnimation(idleAnim);

	    fRandomDriverSluggishness = -0.25f + Random.value*0.5f;

	    fCurrentHealth = fMaxHealth;

	    audioSource = GetComponent<AudioSource>();
    }
	
	// Update is called once per frame
	void Update ()
	{
	    UpdateAvoidTruck();
        UpdateMovement();

	    fChangeDirectionTimer += Time.deltaTime;
	    if (!bAvoidPlayerTruck && fChangeDirectionTimer > 2.0f && Random.value < 0.02f)
	    {
	        vMoveImpetus.x *= (-0.8f - (Random.value*0.4f));
	    }
	}

    void UpdateAvoidTruck()
    {
        if (!bAvoidPlayerTruck)
            return;

        float fDeltaToTruckX = theTruck.transform.position.x - transform.position.x;

        if (Mathf.Abs(fDeltaToTruckX) <= 1.25f + fRandomDriverSluggishness)
        {
            if (fDeltaToTruckX > 0.0f)
                vMoveImpetus.x -= (7.5f + fRandomDriverSluggishness) *Time.deltaTime;
            else
                vMoveImpetus.x += (7.5f + fRandomDriverSluggishness) * Time.deltaTime;
        }
        else
        {
            if (fDeltaToTruckX > 0.0f)
                vMoveImpetus.x += 2.0f * Time.deltaTime;
            else
                vMoveImpetus.x -= 2.0f * Time.deltaTime;
            
        }

        vMoveImpetus.x = Mathf.Clamp(vMoveImpetus.x, -8.0f, 8.0f);
    }

    void UpdateMovement()
    {
        vActualPosition += vMoveImpetus * Time.deltaTime;
        if (theTruck.fBoostValue > 0.0f)
            vActualPosition.y -= theTruck.fBoostValue*Time.deltaTime;

        switch (swoopStateEnum)
        {
            case EEnemySwoopState.swooping_in:

                if (bSwoopFromBehind)
                {
                    if( !theGame.bBoostEngaged)
                        vMoveImpetus.y -= 0.75f * Time.deltaTime;

                    //Let bikes race to catch up and then overshoot
                    if (vActualPosition.y < -1.0f && theGame.bBoostEngaged)
                        vMoveImpetus.y += 2.0f*Time.deltaTime;

                    //Debug.Log("Movespeed Y " + vMoveImpetus.y);
                    if (vActualPosition.y >= fGoalYPosition || vMoveImpetus.y < -1.0f)
                    {
                        swoopStateEnum = EEnemySwoopState.maintaining;
                        fSwoopTimer = 3.0f;
                        vMoveImpetus.y = 0.5f;
                    }
                }
                else
                {
                    vMoveImpetus.y += 1.0f*Time.deltaTime;

                    if (vActualPosition.y <= fGoalYPosition || vMoveImpetus.y > -1.0f)
                    {
                        swoopStateEnum = EEnemySwoopState.maintaining;
                        fSwoopTimer = 3.0f;
                        vMoveImpetus.y = -0.5f;
                    }
                }
                break;
            case EEnemySwoopState.maintaining:
                {
                    vMoveImpetus.y += Time.deltaTime * (bSwoopFromBehind ? -1.0f : 1.0f);

                    fSwoopTimer -= Time.deltaTime;
                    if( fSwoopTimer < 0.0f )
                    {
                        swoopStateEnum = EEnemySwoopState.swooping_out;
                    }
                }
                break;
            case EEnemySwoopState.swooping_out:
                {
                    vMoveImpetus.y += 3.0f * Time.deltaTime * (bSwoopFromBehind ? -1.0f : 1.0f);
                    fSwoopTimer += Time.deltaTime;
                    if (fSwoopTimer > 1.75f)
                    {
                        fSwoopTimer = 0.0f;
                        swoopStateEnum = EEnemySwoopState.swooping_in;

                        vMoveImpetus.y = 8.0f * (bSwoopFromBehind ? 1.0f : -1.0f);

                    }
                }
                break;
        }

        vActualPosition += vImpetusFromOutside*Time.deltaTime;

        vImpetusFromOutside *= 0.095f;
        if (vImpetusFromOutside.sqrMagnitude < 0.05f)
            vImpetusFromOutside = Vector3.zero;

        vActualPosition.x = theGame.KeepXValueInCanyon(vActualPosition.x);

        transform.position = vActualPosition;
    }

    public void TakeDamage(float fDamage)
    {
        fCurrentHealth -= fDamage;
        if (fCurrentHealth <= 0.0f)
            Died();
    }

    public void Died()
    {
        if (bIsDead)
        {
            return;
        }

        bIsDead = true;
        GameObject go = Instantiate(Prefab_DecalOnDeath);
        go.transform.position = transform.position;
        theGame.PlayClip(clip_onDeath);

        theGame.AddScore(1000);

        Destroy(gameObject);
    }

    public void AddForceFromOutside(Vector3 vNewForce)
    {
        vImpetusFromOutside += vNewForce;
    }
}
