﻿using System;
using UnityEngine;
using System.Collections;
using Random = UnityEngine.Random;

public class PlayerTruck : MonoBehaviour
{

    public Vector3 vActualTruckPosition;

    public Vector3 vTruckNavigationImpetus;

    //To give the truck a little natural bobbing and weaving as it moves
    private float fTruckYLocationSineWave;
    public GameObject TruckAnchorObject;

    //Resources
    public float Boost_MaxPower = 1.0f;
    public float Boost_MaxRechargeDelay = 2.0f;

    [HideInInspector]
    public float fBoost_CurrentPower;
    [HideInInspector]
    public float fBoost_RechargeTimer;

    public float Shot_MaxPower = 1.0f;
    [HideInInspector]
    public float fShot_CurrentPower = 1.0f;

    public float Shot_BaseShotSpeedWithOneGun = 1.0f;
    

    public SpriteRenderer Boost_BoostSpriteRenderer;

    public GameManager theGame;
    public EnemyManager theEnemyManager;

    //Boosty boost shit here
    ParticleSystem boostParticleSystem;
    public GameObject arc_boostParticles;

    public float fBoostValue;
    public float fSpeedPenalty;

    public AudioClip clip_ramStart;
    public AudioClip clip_ramStop;

    public AudioClip clip_weaponUpgrade;

    public AudioClip clip_rocketlawnchair;

    public AudioClip clip_hitAndSlow;
    public AudioClip clip_hitAndFast;

    private AudioSource audioSource;

	// Use this for initialization
    void Start()
    {
        theGame = GameObject.Find("GameManager").GetComponent<GameManager>();
        theEnemyManager = GameObject.Find("EnemyManager").GetComponent<EnemyManager>();
        fShot_CurrentPower = 1.0f;

        audioSource = GetComponent<AudioSource>();
    }

    public void StartBoostParticles()
    {
        //if we have no juice, there's no juice.
        if (fBoost_CurrentPower <= 0.0f)
            return;

        fBoost_CurrentPower -= Time.deltaTime;

        //if the boost is off, and we're not going fast, jump up to speed
        if (fBoostValue < 4.0f)
            fBoostValue = 4.2f;
        else
        {
            fBoostValue += 12.0f*Time.deltaTime;
        }

        if (boostParticleSystem == null )
        {
            Debug.Log("Starting New Particles");
            boostParticleSystem = GameObject.Instantiate<GameObject>(arc_boostParticles).GetComponent<ParticleSystem>();
            boostParticleSystem.transform.position = transform.position;
            boostParticleSystem.transform.parent = transform;
            audioSource.PlayOneShot(clip_ramStart);
        }

        //Handle boost delay timer
        fBoost_RechargeTimer = Boost_MaxRechargeDelay;
    }

    public void StopBoostParticles()
    {
        if (boostParticleSystem != null )
        {
            boostParticleSystem.Stop();
            Destroy(boostParticleSystem.gameObject,0.1f);
            boostParticleSystem = null;

            audioSource.Stop();

            if( fBoost_CurrentPower <= 0.1f)
                audioSource.PlayOneShot(clip_ramStop);
        }
    }
	
	// Update is called once per frame
	void Update ()
	{
	    if (fSpeedPenalty > 0.0f)
	    {
	        fSpeedPenalty -= Time.deltaTime;
	    }

	    fSpeedPenalty = Mathf.Clamp(fSpeedPenalty, 0.0f, fSpeedPenalty);

        UpdateBoostLogic();
        UpdateShotLogic();

	    fTruckYLocationSineWave += (Time.deltaTime * 0.25f) * (0.7f + Random.value *0.6f);
	    if (fTruckYLocationSineWave > 1.0f)
	        fTruckYLocationSineWave -= 1.0f;

	    NavigateViaImpetus();
	    KeepTruckInBounds();
	    VibrateTruck();
	}

    void KeepTruckInBounds()
    {
        vActualTruckPosition.x = theGame.KeepXValueInCanyon(vActualTruckPosition.x);
    }

    void UpdateBoostLogic()
    {
        //Decay boost speed
        if (fBoostValue > 0.0f)
            fBoostValue -= 8.0f * Time.deltaTime;

        fBoostValue = Mathf.Clamp(fBoostValue, 0.0f, fBoostValue);

        //tick down recharge timer
        float fRechargeTimerDelta = Time.deltaTime;
        if (theGame.bShotEngaged)
            fRechargeTimerDelta *= 4.0f;

        float fRechargePowerDelta = Time.deltaTime * 0.25f;
        if (theGame.bShotEngaged)
            fRechargePowerDelta *= 4.0f;


        if (fBoost_RechargeTimer > 0.0f)
            fBoost_RechargeTimer -= fRechargeTimerDelta;

        if (fBoost_RechargeTimer <= 0.0f)
            fBoost_CurrentPower += fRechargePowerDelta; 

        if (fBoost_CurrentPower > Boost_MaxPower)
            fBoost_CurrentPower = Boost_MaxPower;

        if (theGame.bBoostEngaged)
        {
            Boost_BoostSpriteRenderer.enabled = true;
            float fColorSin = fBoost_CurrentPower/Boost_MaxPower;
            Boost_BoostSpriteRenderer.color = Color.Lerp(Color.red, Color.yellow, fColorSin);

            Boost_BoostSpriteRenderer.transform.localScale = new Vector3( Random.value * 0.4f + 0.8f, Random.value * 0.4f + 0.8f, 1.0f);
        }
        else
        {
            Boost_BoostSpriteRenderer.enabled = false;
        }
    }

    void UpdateShotLogic()
    {
        //tick down recharge timer
        float fRechargePowerDelta = Time.deltaTime * 0.18f;
        if (theGame.bShotEngaged)
            fRechargePowerDelta = 0.0f;

        if (theGame.bBoostEngaged)
            fRechargePowerDelta *= 8.0f;

        fShot_CurrentPower += fRechargePowerDelta;

        if (fShot_CurrentPower > Shot_MaxPower)
            fShot_CurrentPower = Shot_MaxPower;
    }

    void NavigateViaImpetus()
    {
        vActualTruckPosition += vTruckNavigationImpetus*Time.deltaTime;

        //float back toward center
        float fImpetusXVal = 0.5f;
        if (theGame.bBoostEngaged)
            fImpetusXVal = 2.0f;

        if( vActualTruckPosition.x > 0.25f )
            vTruckNavigationImpetus = new Vector3(fImpetusXVal * -1.0f, 0.0f, 0.0f);
        else if (vActualTruckPosition.x < -0.25f)
            vTruckNavigationImpetus = new Vector3(fImpetusXVal, 0.0f, 0.0f);

    }

    void VibrateTruck()
    {
        Vector3 vTruckPosition = vActualTruckPosition;

        //shake
        vTruckPosition.x += (-0.02f + 0.04f*Random.value) * ( fSpeedPenalty > 0.0f ? 3.0f : 1.0f);
        vTruckPosition.y += (-0.05f + 0.1f * Random.value) * (fSpeedPenalty > 0.0f ? 3.0f : 1.0f);

        //sine wavey power
        vTruckPosition.y += 0.5f * (Mathf.Sin(fTruckYLocationSineWave * 6.28f));

        //boost energy
        if (fBoostValue > 4.0f)
            vTruckPosition.y += 0.25f* (fBoostValue - 4.0f);


        transform.position = vTruckPosition;
        TruckAnchorObject.transform.localPosition = new Vector3(0.0f, -0.02f + 0.04f * Random.value, 0.0f);

    }

    void OnTriggerStay2D( Collider2D coll )
    {
        if (theGame.bBoostEngaged)
            return;

        if (theGame.IsGameOver())
            return;

        GameObject collisionObject = coll.gameObject;

        if (collisionObject.tag == "ObstacleNavigation")
        {
            Vector3 vColliderPosition = collisionObject.transform.position;

            Vector3 vOurPosition = transform.position;

            float xDelta = vOurPosition.x - vColliderPosition.x;


            //Bad Hax
            if (xDelta < 0)
                vTruckNavigationImpetus.x = -2.0f;
            else if (xDelta > 0)
                vTruckNavigationImpetus.x = 2.0f;
        }

    }

    void OnTriggerEnter2D(Collider2D hitObject )
    {
        if (theGame.IsGameOver())
            return;

        Debug.Log("I hit " + hitObject.gameObject + " with tag " + hitObject.gameObject.tag);

        if (hitObject.gameObject.tag == "ObstacleImpact")
        {
            //is it actually on obstacle?
            ObstacleBehavior obstacle = hitObject.gameObject.GetComponent<ObstacleBehavior>();

            if (obstacle == null)
                obstacle = hitObject.gameObject.GetComponentInParent<ObstacleBehavior>();

            if (obstacle != null)
            {
                obstacle.HitAndDie();

                if (theGame.bBoostEngaged)
                {
                    theGame.AddScore(200);
                    audioSource.PlayOneShot(clip_hitAndFast);
                    theGame.BaselineTruckSpeed += 0.1f;
                }
                else
                {
                    audioSource.PlayOneShot(clip_hitAndSlow);
                    fSpeedPenalty += 1.0f;
                }
                return;
            }
        }
        else if(hitObject.gameObject.tag == "Enemy" )
        {
            EnemyBase eb = hitObject.gameObject.GetComponent<EnemyBase>();
            if( eb == null )
                eb = hitObject.gameObject.GetComponentInParent<EnemyBase>();

            if (eb != null)
            {
                eb.TakeDamage(1000);
                if (theGame.bBoostEngaged)
                {
                    audioSource.PlayOneShot(clip_hitAndFast);
                    theGame.BaselineTruckSpeed += 0.1f;
                }
                else
                {
                    audioSource.PlayOneShot(clip_hitAndSlow);
                    fSpeedPenalty += 0.5f;
                }
            }
        }

    }

    public void ActivateMoreTurrets()
    {
        TruckGunManager tgm = GetComponentInChildren<TruckGunManager>();
        tgm.ExtraTurret_01.SetActive(true);
        tgm.ExtraTurret_02.SetActive(true);
        theGame.PlayClip(clip_weaponUpgrade);
        fShot_CurrentPower = Shot_MaxPower;
        fBoost_CurrentPower = Boost_MaxPower;

        theEnemyManager.KillAll();
    }

    public void ActivateFlameThrowers()
    {
        TruckGunManager tgm = GetComponentInChildren<TruckGunManager>();
        tgm.FlameThrower_01.SetActive(true);
        tgm.FlameThrower_02.SetActive(true);
        theGame.PlayClip(clip_weaponUpgrade);
        Shot_MaxPower += 0.25f;
        fShot_CurrentPower = Shot_MaxPower;
        fBoost_CurrentPower = Boost_MaxPower;
    }

    public void ActivateRocketLawnchair()
    {
        TruckGunManager tgm = GetComponentInChildren<TruckGunManager>();
        tgm.RocketLawnchair_01.SetActive(true);
        tgm.RocketLawnchair_02.SetActive(true);
        theGame.PlayClip(clip_rocketlawnchair);
        Shot_MaxPower += 0.25f;
        fShot_CurrentPower = Shot_MaxPower;
        fBoost_CurrentPower = Boost_MaxPower;
    }


}
