﻿using UnityEngine;
using System.Collections;

public class Projectile : MonoBehaviour 
{
    public Vector3 vDirection;
    public float ProjectileSpeed;

    [HideInInspector]
    public float fSpeed;

    public float SizeDeltaOverTime;

    public float fDamage = 1.0f;

    public GameObject Prefab_Impact;

    public bool bAlwaysRotateUp;

    public float fDestroyAfter;

    public bool DontDestroyOnHit;
    public float DamageEveryThisOften;

    private bool bPayloadSpent;
    
    private float fDeliverPayloadTick;

    protected GameObject targetObject;

    protected PlayerTruck theTruck;
    protected EnemyManager theEnemyManager;

    public AudioClip clip_launchClip;
    public AudioClip clip_impactClip;
    protected  AudioSource audioSource;

    // Use this for initialization
	public virtual void Start ()
	{
	    if (fDestroyAfter > 0.0f)
	        Destroy(gameObject, fDestroyAfter);
	    else
	        Destroy(gameObject, 20.0f);

	    Init();
	}

    public virtual void Init()
    {
        theTruck = GameObject.Find("PlayerTruck").GetComponent<PlayerTruck>();
        theEnemyManager = GameObject.Find("EnemyManager").GetComponent<EnemyManager>();
        audioSource = GetComponent<AudioSource>();
    }

	// Update is called once per frame
	public virtual void Update () 
    {
        UpdateMovement();

	    if (bAlwaysRotateUp)
	    {
            transform.right = Vector3.Cross(Vector3.up, new Vector3(0, 0, 1));
	    }
        else
            transform.right = Vector3.Cross(vDirection, new Vector3(0, 0, 1));

	    if (SizeDeltaOverTime != 0.0f)
	    {
	        Vector3 vScale = transform.localScale;
	        vScale.x += vScale.x*SizeDeltaOverTime*Time.deltaTime;
	        vScale.y += vScale.y*SizeDeltaOverTime*Time.deltaTime;
	        transform.localScale = vScale;
	    }

	    if (DontDestroyOnHit)
	    {
	        fDeliverPayloadTick -= Time.deltaTime;
	    }
	}

    public virtual void UpdateMovement()
    {
        Vector3 vPos = transform.position;
        vPos += vDirection * fSpeed * Time.deltaTime;

        transform.position = vPos;
    }

    public virtual void Launch()
    {
        Init();

        if( clip_launchClip != null)
            audioSource.PlayOneShot(clip_launchClip);
    }

    public virtual void OnTriggerEnter2D(Collider2D hitObject)
    {
        Debug.Log("OMG Projectile hit a " + hitObject.gameObject.name);
        EnemyBase eb = hitObject.gameObject.GetComponent<EnemyBase>();
        if (eb != null)
        {
            eb.TakeDamage(fDamage);
            eb.AddForceFromOutside( transform.up * 10.0f);
        }

        GameObject go = Instantiate(Prefab_Impact);
        go.transform.position = transform.position;
        Destroy(go, 3.0f);
        bPayloadSpent = true;

        if (!DontDestroyOnHit)
        {
            Destroy(gameObject);
        }
    }

    public virtual void OnTriggerStay2D(Collider2D hitObject)
    {
        if (bPayloadSpent)
            return;

        if (fDeliverPayloadTick > 0.0f)
            return;

        EnemyBase eb = hitObject.gameObject.GetComponent<EnemyBase>();
        if (eb != null)
        {
            eb.TakeDamage(fDamage);
            eb.AddForceFromOutside(transform.up * 10.0f);
        }

        fDeliverPayloadTick = DamageEveryThisOften;
        GameObject go = Instantiate(Prefab_Impact);
        go.transform.position = transform.position;
        Destroy(go, 3.0f);

    }
}
