﻿using UnityEngine;
using System.Collections;

public class Projectile_MultiStageRocket : Projectile
{

    bool bInTrackingStage;

    float fTimeInTrackingStage = 2.0f;

    public AudioClip clip_launchNextStage;

    public GameObject Prefab_StageChangeTrail;

	public override void Update () 
    {
        base.Update();

        if (!bInTrackingStage)
            return;

        fTimeInTrackingStage -= Time.deltaTime;

        //Switch to firing stage
        if( fTimeInTrackingStage <= 0.0f)
        {
            bInTrackingStage = false;
            fSpeed = 20.0f;
            vDirection = new Vector3(0, 1, 0);
            bAlwaysRotateUp = false;

            if( clip_launchNextStage != null)
            {
                audioSource.PlayOneShot(clip_launchNextStage);
            }

            EnemyBase eb = theEnemyManager.GimmeSomethingToKill(transform.position, new Vector3(0, 1, 0), true);
            if( eb!= null )
            {
                targetObject = eb.gameObject;
                vDirection = (targetObject.transform.position - transform.position).normalized;
            }

            if( Prefab_StageChangeTrail != null )
            {
                GameObject tail = Instantiate(Prefab_StageChangeTrail);
                tail.transform.position = transform.position;
                tail.transform.parent = transform;
            }
        }
	}

    public override void UpdateMovement()
    {
        if (!bInTrackingStage)
            fSpeed += 10.0f * Time.deltaTime;

        base.UpdateMovement();
    }
    public override void OnTriggerEnter2D(Collider2D hitObject)
    {
        if (!bInTrackingStage)
            base.OnTriggerEnter2D(hitObject);
    }

    public override void OnTriggerStay2D(Collider2D hitObject)
    {
        if (!bInTrackingStage)
            base.OnTriggerStay2D(hitObject);
    }

    public override void Launch()
    {
        base.Launch();

        float fXDelta = (transform.position - theTruck.transform.position).x;
        if (fXDelta > 0.0f)
        {
            vDirection = new Vector3(1, 0.4f, 0);
        }
        else
            vDirection = new Vector3(-1, 0.4f, 0);

        vDirection.Normalize();
        fSpeed = 0.5f + Random.value * 1.5f;
        bInTrackingStage = true;

        
    }
}
