﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

[System.Serializable]
public class ShepAnim
{
    public string Name;
    public List<Sprite> Sprites;
    public float FrameDelay;
}

public class ShepSpriteAnimator : MonoBehaviour 
{
    [HideInInspector]
    public ShepAnim currentAnimation;

    public ShepAnim playOnStartAnim;
    public SpriteRenderer spriteRenderer;

    public AudioClip clip_animStart;
    public AudioClip clip_animEnd;

    private AudioSource audioSource;

    public bool DestroyOnFinish;

    float fTimer;
    int iFrameIndex;
    

	// Use this for initialization
	void Start ()
	{
	    audioSource = GetComponent<AudioSource>();
        fTimer = 0;
        if (currentAnimation != null)
            fTimer = currentAnimation.FrameDelay;

        if( playOnStartAnim != null && playOnStartAnim.Sprites != null && playOnStartAnim.Sprites.Count > 0)
            SetAnimation(playOnStartAnim);

	}
	
    public void SetAnimation( ShepAnim newAnim )
    {
        currentAnimation = newAnim;
        if (currentAnimation != null)
            fTimer = currentAnimation.FrameDelay;

        iFrameIndex = 0;

        if( clip_animStart != null && audioSource != null )
            audioSource.PlayOneShot(clip_animStart);
    }

	// Update is called once per frame
	void Update () 
    {
        if (currentAnimation == null)
            return;

        fTimer -= Time.deltaTime;
        if( fTimer < 0 )
        {
            fTimer = currentAnimation.FrameDelay;

            iFrameIndex++;
            if (iFrameIndex >= currentAnimation.Sprites.Count)
            {
                iFrameIndex = 0;
                if (DestroyOnFinish)
                {
                    spriteRenderer.enabled = false;
                    Destroy(gameObject,1.0f);
                    return;
                }
            }

            spriteRenderer.sprite= currentAnimation.Sprites[iFrameIndex];
        }
	}
}
