﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TruckGunManager : MonoBehaviour
{

    public List<TruckWeapon> allGuns;

    private int iShootingIndex = 0;
    private float fDelayBetweenShots;
    private float fShotTimer;

    private PlayerTruck theTruck;
    private EnemyManager theEnemyManager;
    private GameManager theGame;


    public GameObject ExtraTurret_01;
    public GameObject ExtraTurret_02;

    public GameObject FlameThrower_01;
    public GameObject FlameThrower_02;

    public GameObject RocketLawnchair_01;
    public GameObject RocketLawnchair_02;

	// Use this for initialization
	void Start () 
    {
        theTruck = GameObject.Find("PlayerTruck").GetComponent<PlayerTruck>();
        theEnemyManager = GameObject.Find("EnemyManager").GetComponent<EnemyManager>();
	    theGame = GameObject.Find("GameManager").GetComponent<GameManager>();

        ExtraTurret_01.SetActive(false);
        ExtraTurret_02.SetActive(false);
        FlameThrower_01.SetActive(false);
        FlameThrower_02.SetActive(false);
        RocketLawnchair_01.SetActive(false);
        RocketLawnchair_02.SetActive(false);

    }

    void UpdateGunsOnMe()
    {
        allGuns = new List<TruckWeapon>( GetComponentsInChildren<TruckWeapon>());

        //track badguys?

        //debug crapola
        foreach (TruckWeapon checkGun in allGuns)
        {
            //can we track our current guy?
            if (checkGun.CurrentlyTrackedEnemy != null)
            {
                if (!checkGun.IsTargetInFiringArc( checkGun.CurrentlyTrackedEnemy.transform.position))
                    checkGun.CurrentlyTrackedEnemy = null;

            }

            //get one if we need one
            if (checkGun.CurrentlyTrackedEnemy == null)
            {
                float fBestDist = float.MaxValue;
                EnemyBase bestFoe = null;
                foreach (EnemyBase maybeEnemy in theEnemyManager.allEnemies)
                {
                    if (maybeEnemy == null)
                        continue;

                    if( !checkGun.IsTargetInFiringArc(maybeEnemy.transform.position))
                        continue;

                    float dist = (maybeEnemy.transform.position - transform.position).sqrMagnitude;
                    if (dist < fBestDist)
                    {
                        fBestDist = dist;
                        bestFoe = maybeEnemy;
                    }

                }

                if( bestFoe != null )
                    checkGun.CurrentlyTrackedEnemy = bestFoe.gameObject;
            }
        }
    }
	
	// Update is called once per frame
	void Update ()
	{
	    UpdateGunsOnMe();

	    if (allGuns == null || allGuns.Count == 0)
	        return;

	    if (!theGame.IsShootBeingMashed())
	        return;

	    fDelayBetweenShots = theTruck.Shot_BaseShotSpeedWithOneGun/allGuns.Count;

        //Time between shots moves faster if there is more ammo in the tank
	    float fRatio = theTruck.fShot_CurrentPower/theTruck.Shot_MaxPower;
        
	    float fDeltaplier = theTruck.fShot_CurrentPower/theTruck.Shot_MaxPower * 4.0f;

        //Even faster in the blue glowy range
	    if (fRatio >= 0.7f)
	        fDeltaplier *= 5.0f;
	    else //limit speed when not blue glowy
	    {
	        if (fDelayBetweenShots < 0.2f)
	            fDelayBetweenShots = 0.2f;
	    }

	    fShotTimer += Time.deltaTime + (Time.deltaTime * fDeltaplier);

        if (fShotTimer > fDelayBetweenShots)
	    {
	        //shoot a thing
	        if (iShootingIndex >= allGuns.Count)
	            iShootingIndex = 0;

	        if (allGuns[iShootingIndex].HasEnoughChargeToFire(theTruck.fShot_CurrentPower))
	        {
	            allGuns[iShootingIndex].StartBurst();
	            fShotTimer = 0.0f;
	        }

	        iShootingIndex++;
	    }

	    theTruck.fShot_CurrentPower -= Time.deltaTime;
	    if (theTruck.fShot_CurrentPower < 0.0f)
	        theTruck.fShot_CurrentPower = 0.0f;
	}

  
}
