﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TruckWeapon : MonoBehaviour
{

    public GameObject Prefab_Projectile;

    public GameObject CurrentlyTrackedEnemy;

    private Vector3 vFacing;

    public bool bShouldTrackTarget;

    public bool bLimitedFireArc;
    public Vector3 vFireArcMainVector;

    public bool bIsFiring;
    public int ShotsPerBurst;
    public float DelayBetweenShots;
    private int iShotsLeftInBurst;
    private float fShotDelay;

    public List<float> AnglesToFireMultipleProjectilesFrom;

    public GameManager theGame;

    public bool bAlwaysRotateUp;

    public float fRequireThisMuchChargeToFire;

    // Use this for initialization
    private void Start()
    {
        theGame = GameObject.Find("GameManager").GetComponent<GameManager>();
    }

    // Update is called once per frame
    private void Update()
    {
        TrackTarget();
        if (bIsFiring)
            TickDownShots();
    }

    public void StartBurst()
    {
        Fire();
        iShotsLeftInBurst--;
        if (iShotsLeftInBurst <= 0)
        {
            ResetBurst();
            bIsFiring = false;
        }
        else
        {
            bIsFiring = true;
        }
    }

    private void ResetBurst()
    {
        iShotsLeftInBurst = ShotsPerBurst;
        fShotDelay = 0.0f;
    }

    private void TickDownShots()
    {
        fShotDelay -= Time.deltaTime;
        if (fShotDelay <= 0.0f)
        {
            fShotDelay = DelayBetweenShots;
            Fire();
            iShotsLeftInBurst--;
            if (iShotsLeftInBurst <= 0)
            {
                ResetBurst();
                bIsFiring = false;
            }
        }
    }

    private void TrackTarget()
    {
        vFireArcMainVector.Normalize();
        if (CurrentlyTrackedEnemy != null && bShouldTrackTarget)
        {
            Vector3 towardsTarget = CurrentlyTrackedEnemy.transform.position - transform.position;
            towardsTarget.Normalize();
            transform.right = Vector3.Cross(towardsTarget, new Vector3(0, 0, 1));
            Debug.DrawLine(transform.position, CurrentlyTrackedEnemy.transform.position, Color.red, 0.25f);

        }
        else
        {
            transform.right = Vector3.Cross(vFireArcMainVector, new Vector3(0, 0, 1));
        }
    }

    public bool IsTargetInFiringArc(Vector3 vTargetPosition)
    {
        if (vTargetPosition.y < -4)
            return false;

        if (!bLimitedFireArc)
            return true;

        return Vector3.Dot(vFireArcMainVector, vTargetPosition) > 0.0f;
    }

    private void Fire()
    {
        FireOneShot(0f);
        if (AnglesToFireMultipleProjectilesFrom != null && AnglesToFireMultipleProjectilesFrom.Count > 0)
        {
            foreach (float f in AnglesToFireMultipleProjectilesFrom)
                FireOneShot(f);
        }
        fShotDelay = DelayBetweenShots;
    }

    private void FireOneShot(float deviateAngle)
    {
        GameObject go = Instantiate(Prefab_Projectile);
        Projectile p = go.GetComponent<Projectile>();

        p.transform.position = transform.position;
        p.fSpeed = p.ProjectileSpeed;

        Vector3 towardsTarget = transform.up.normalized;

        if (CurrentlyTrackedEnemy != null && bShouldTrackTarget)
        {
            towardsTarget = CurrentlyTrackedEnemy.transform.position - transform.position;
            towardsTarget.Normalize();

        }

        if (deviateAngle != 0)
        {
            Vector2 vRotMe = towardsTarget;
            vRotMe = theGame.Rotate2DVector(vRotMe, deviateAngle);
            towardsTarget.x = vRotMe.x;
            towardsTarget.y = vRotMe.y;
        }

        p.vDirection = towardsTarget;
        p.Launch();
    }

    public bool HasEnoughChargeToFire(float fCharge)
    {
        return fCharge > fRequireThisMuchChargeToFire;
    }
}

/* 

Good code for tracking with 2D turrets

 *     public override void UpdateMotion()
    {
        Vector3 towardsPlayer = playerScript.transform.position - transform.position;
        towardsPlayer.Normalize();
        transform.right = Vector3.Cross(towardsPlayer, new Vector3(0, 0, 1));
    }

    public override void StayVertical()
    {
       
    }

    public override void UpdateAttacks()
    {
        Vector3 towardsPlayer = playerScript.transform.position - transform.position;

        towardsPlayer.x += -0.2f + Random.value*0.4f;
        towardsPlayer.Normalize();

        for (int t = 0; t < Attacks.Count(); t++)
        {
            Attacks[t].Update();

            if (transform.position.x < 12.0f)
            {
                float fShotSpeed = 15.0f + (5.5f * mcs.GetEnemySpeedMultiplier());
                Attacks[t].TryFire(transform.position, towardsPlayer, fShotSpeed);
            }
        }
    }



*/